@@ -546,7 +546,7 @@ void KMainWindow::closeEvent(QCloseEvent *e)
     // Delete the marker that says we don't want to restore the position of the
     // next-opened instance; now that a window is closing, we do want to do this
     if (d->autoSaveGroup.isValid()) {
-        d->autoSaveGroup.deleteEntry("RestorePositionForNextInstance");
+        d->getValidStateConfig(d->autoSaveGroup).deleteEntry("RestorePositionForNextInstance");
     }
     d->_k_slotSaveAutoSavePosition();
 
@@ -560,8 +560,9 @@ void KMainWindow::closeEvent(QCloseEvent *e)
         e->ignore(); // if the window should not be closed, don't close it
     }
     // If saving session, we are processing a fake close event, and might get the real one later.
-    if (e->isAccepted() && qApp->isSavingSession())
+    if (e->isAccepted() && qApp->isSavingSession()) {
         d->suppressCloseEvent = true;
+    }
 }
 
 bool KMainWindow::queryClose()
@@ -603,16 +604,17 @@ void KMainWindow::saveMainWindowSettings(KConfigGroup &cg)
     Q_D(KMainWindow);
     // qDebug(200) << "KMainWindow::saveMainWindowSettings " << cg.name();
 
+    KConfigGroup stateConfig = d->getValidStateConfig(cg);
     // Called by session management - or if we want to save the window size anyway
     if (d->autoSaveWindowSize) {
-        KWindowConfig::saveWindowSize(windowHandle(), cg);
-        KWindowConfig::saveWindowPosition(windowHandle(), cg);
+        KWindowConfig::saveWindowSize(windowHandle(), stateConfig);
+        KWindowConfig::saveWindowPosition(windowHandle(), stateConfig);
     }
 
     // One day will need to save the version number, but for now, assume 0
     // Utilise the QMainWindow::saveState() functionality.
     const QByteArray state = saveState();
-    cg.writeEntry("State", state.toBase64());
+    stateConfig.writeEntry("State", state.toBase64());
 
     QStatusBar *sb = internalStatusBar(this);
     if (sb) {
@@ -686,16 +688,21 @@ bool KMainWindow::readPropertiesInternal(KConfig *config, int number)
     return true;
 }
 
-void KMainWindow::applyMainWindowSettings(const KConfigGroup &cg)
+void KMainWindow::applyMainWindowSettings(const KConfigGroup &_cg)
 {
     Q_D(KMainWindow);
     // qDebug(200) << "KMainWindow::applyMainWindowSettings " << cg.name();
 
+    KConfigGroup cg = _cg;
+    d->migrateStateDataIfNeeded(cg);
+
     QWidget *focusedWidget = QApplication::focusWidget();
 
     const bool oldLetDirtySettings = d->letDirtySettings;
     d->letDirtySettings = false;
 
+    KConfigGroup stateConfig = d->getValidStateConfig(cg);
+
     if (!d->sizeApplied && (!window() || window() == this)) {
         winId(); // ensure there's a window created
         // Set the window's size from the existing widget geometry to respect the
@@ -704,7 +711,7 @@ void KMainWindow::applyMainWindowSettings(const KConfigGroup &cg)
         // TODO: remove once QTBUG-40584 is fixed; see below
         windowHandle()->setWidth(width());
         windowHandle()->setHeight(height());
-        KWindowConfig::restoreWindowSize(windowHandle(), cg);
+        KWindowConfig::restoreWindowSize(windowHandle(), stateConfig);
         // NOTICE: QWindow::setGeometry() does NOT impact the backing QWidget geometry even if the platform
         // window was created -> QTBUG-40584. We therefore copy the size here.
         // TODO: remove once this was resolved in QWidget QPA
@@ -716,13 +723,12 @@ void KMainWindow::applyMainWindowSettings(const KConfigGroup &cg)
         KSharedConfigPtr config = KSharedConfig::openConfig();
         KConfigGroup group(config, "General");
         if (group.readEntry("AllowKDEAppsToRememberWindowPositions", true)) {
-            if (cg.readEntry("RestorePositionForNextInstance", true)) {
+            if (stateConfig.readEntry("RestorePositionForNextInstance", true)) {
                 KWindowConfig::restoreWindowPosition(windowHandle(), cg);
                 // Save the fact that we now don't want to restore position
                 // anymore; if we did, the next instance would completely cover
                 // the existing one
-                KConfigGroup cgWritable = cg; // because cg is const
-                cgWritable.writeEntry("RestorePositionForNextInstance", false);
+                stateConfig.writeEntry("RestorePositionForNextInstance", false);
             }
         }
     }
@@ -757,9 +763,9 @@ void KMainWindow::applyMainWindowSettings(const KConfigGroup &cg)
         n++;
     }
 
-    QByteArray state;
-    if (cg.hasKey("State")) {
-        state = cg.readEntry("State", state);
+    if (stateConfig.hasKey("State")) {
+        QByteArray state;
+        state = stateConfig.readEntry("State", state);
         state = QByteArray::fromBase64(state);
         // One day will need to load the version number, but for now, assume 0
         restoreState(state);
@@ -850,6 +856,18 @@ KConfigGroup KMainWindow::autoSaveConfigGroup() const
     return d->autoSaveSettings ? d->autoSaveGroup : KConfigGroup();
 }
 
+void KMainWindow::setStateConfigGroup(const QString &configGroup)
+{
+    Q_D(KMainWindow);
+    d->m_stateConfigGroup = KSharedConfig::openStateConfig()->group(configGroup);
+}
+
+KConfigGroup KMainWindow::stateConfigGroup() const
+{
+    Q_D(const KMainWindow);
+    return d->m_stateConfigGroup;
+}
+
 void KMainWindow::saveAutoSaveSettings()
 {
     Q_D(KMainWindow);
@@ -971,10 +989,11 @@ QList<KToolBar *> KMainWindow::toolBars() const
     QList<KToolBar *> ret;
 
     const auto theChildren = children();
-    for (QObject *child : theChildren)
+    for (QObject *child : theChildren) {
         if (KToolBar *toolBar = qobject_cast<KToolBar *>(child)) {
             ret.append(toolBar);
         }
+    }
 
     return ret;
 }
