@@ -1,29 +1,15 @@
 /*
     This file is part of the KDE libraries
-     Copyright
-     (C) 2000 Reginald Stadlbauer (reggie@kde.org)
-     (C) 1997 Stephan Kulow (coolo@kde.org)
-     (C) 1997-2000 Sven Radej (radej@kde.org)
-     (C) 1997-2000 Matthias Ettrich (ettrich@kde.org)
-     (C) 1999 Chris Schlaeger (cs@kde.org)
-     (C) 2002 Joseph Wenninger (jowenn@kde.org)
-     (C) 2005-2006 Hamish Rodda (rodda@kde.org)
-     (C) 2000-2008 David Faure (faure@kde.org)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License version 2 as published by the Free Software Foundation.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    SPDX-FileCopyrightText: 2000 Reginald Stadlbauer <reggie@kde.org>
+    SPDX-FileCopyrightText: 1997 Stephan Kulow <coolo@kde.org>
+    SPDX-FileCopyrightText: 1997-2000 Sven Radej <radej@kde.org>
+    SPDX-FileCopyrightText: 1997-2000 Matthias Ettrich <ettrich@kde.org>
+    SPDX-FileCopyrightText: 1999 Chris Schlaeger <cs@kde.org>
+    SPDX-FileCopyrightText: 2002 Joseph Wenninger <jowenn@kde.org>
+    SPDX-FileCopyrightText: 2005-2006 Hamish Rodda <rodda@kde.org>
+    SPDX-FileCopyrightText: 2000-2008 David Faure <faure@kde.org>
 
+    SPDX-License-Identifier: LGPL-2.0-only
 */
 
 #ifndef KMAINWINDOW_H
@@ -32,6 +18,7 @@
 #include <kxmlgui_export.h>
 
 #include <QMainWindow>
+#include <memory>
 
 class QMenu;
 class KConfig;
@@ -40,12 +27,6 @@ class KMWSessionManager;
 class KMainWindowPrivate;
 class KToolBar;
 
-// internal, not public API, may change any time
-#define XMLGUI_DECLARE_PRIVATE(classname) \
-    inline classname ## Private *k_func() { return reinterpret_cast<classname ## Private *>(k_ptr); } \
-    inline const classname ## Private *k_func() const { return reinterpret_cast<classname ## Private *>(k_ptr); } \
-    friend class classname ## Private;
-
 #if KXMLGUI_ENABLE_DEPRECATED_SINCE(5, 65)
 /**
  * @relates KMainWindow
@@ -104,14 +85,14 @@ class KToolBar;
  * the last mainwindow will quit the application unless there is still something
  * that holds a ref in KGlobal - like a KIO job, or a systray icon.
  *
- * @author Reginald Stadlbauer (reggie@kde.org) Stephan Kulow (coolo@kde.org), Matthias Ettrich (ettrich@kde.org), Chris Schlaeger (cs@kde.org), Sven Radej (radej@kde.org). Maintained by David Faure (faure@kde.org)
+ * @author Reginald Stadlbauer (reggie@kde.org) Stephan Kulow (coolo@kde.org), Matthias Ettrich (ettrich@kde.org), Chris Schlaeger (cs@kde.org), Sven Radej
+ * (radej@kde.org). Maintained by David Faure (faure@kde.org)
  */
 
 class KXMLGUI_EXPORT KMainWindow : public QMainWindow
 {
     friend class KMWSessionManager;
     friend class DockResizeListener;
-    XMLGUI_DECLARE_PRIVATE(KMainWindow)
     Q_OBJECT
     Q_PROPERTY(bool hasMenuBar READ hasMenuBar)
     Q_PROPERTY(bool autoSaveSettings READ autoSaveSettings)
@@ -195,8 +176,7 @@ public:
      * @deprecated Since 5.0, use KHelpMenu directly
      */
     KXMLGUI_DEPRECATED_VERSION(5, 0, "Use KHelpMenu directly")
-    QMenu *helpMenu(const QString &aboutAppText = QString(),
-                    bool showWhatsThis = true);
+    QMenu *helpMenu(const QString &aboutAppText = QString(), bool showWhatsThis = true);
 #endif
 
 #if KXMLGUI_ENABLE_DEPRECATED_SINCE(5, 0)
@@ -340,16 +320,14 @@ public:
      *   "virtual QSize sizeHint() const;" to specify a default size rather
      *   than letting QWidget::adjust use the default size of 0x0.
      */
-    void setAutoSaveSettings(const QString &groupName = QStringLiteral("MainWindow"),
-                             bool saveWindowSize = true);
+    void setAutoSaveSettings(const QString &groupName = QStringLiteral("MainWindow"), bool saveWindowSize = true);
 
     /**
      * Overload that lets you specify a KConfigGroup.
      * This allows the settings to be saved into another file than KSharedConfig::openConfig().
      * @since 4.1
      */
-    void setAutoSaveSettings(const KConfigGroup &group,
-                             bool saveWindowSize = true);
+    void setAutoSaveSettings(const KConfigGroup &group, bool saveWindowSize = true);
 
     /**
      * Disable the auto-save-settings feature.
@@ -382,6 +360,23 @@ public:
      */
     KConfigGroup autoSaveConfigGroup() const;
 
+    /**
+     * Set the config group name for state config returned by @p stateConfigGroup
+     * If this method is called the window size and window state are stored in the resulting KConfigGroup.
+     *
+     * @see KSharedConfig::openStateConfig
+     *
+     * @since 5.88
+     */
+    void setStateConfigGroup(const QString &configGroup);
+
+    /*
+     * Returns the KConfigGroup used to store state data like window sizes or window state.
+     * The resulting group in invalid if @p setStateConfig is not called explicitly.
+     * @since 5.88
+     */
+    KConfigGroup stateConfigGroup() const;
+
     /**
      * Read settings for statusbar, menubar and toolbar from their respective
      * groups in the config file and apply them.
@@ -413,7 +408,10 @@ public:
      * @deprecated since 5.0, the functionality got removed
      **/
     KXMLGUI_DEPRECATED_VERSION(5, 0, "Remove usage, is a no-op now")
-    bool initialGeometrySet() const { return false; }
+    bool initialGeometrySet() const
+    {
+        return false;
+    }
 #endif
 
 public Q_SLOTS:
@@ -537,14 +535,18 @@ protected:
      * in this function!
      *
      */
-    virtual void saveProperties(KConfigGroup &) {}
+    virtual void saveProperties(KConfigGroup &)
+    {
+    }
 
     /**
      * Read your instance-specific properties.
      *
      * Is called indirectly by restore().
      */
-    virtual void readProperties(const KConfigGroup &) {}
+    virtual void readProperties(const KConfigGroup &)
+    {
+    }
 
     /**
       * Save your application-wide properties. The function is
@@ -598,32 +600,34 @@ protected:
 protected Q_SLOTS:
 #if KXMLGUI_BUILD_DEPRECATED_SINCE(5, 0)
     /**
-    * This slot does nothing.
-    *
-    * It must be reimplemented if you want
-    * to use a custom About Application dialog box. This slot is
-    * connected to the About Application entry in the menu returned
-    * by customHelpMenu.
-    *
-    * Example:
-    * \code
-    *
-    * void MyMainLevel::setupInterface()
-    * {
-    *   ..
-    *   menuBar()->addMenu( customHelpMenu() );
-    *   ..
-    * }
-    *
-    * void MyMainLevel::showAboutApplication()
-    * {
-    *   <activate your custom dialog>
-    * }
-    * \endcode
-    * @deprecated Since 5.0, use KHelpMenu
-    */
+     * This slot does nothing.
+     *
+     * It must be reimplemented if you want
+     * to use a custom About Application dialog box. This slot is
+     * connected to the About Application entry in the menu returned
+     * by customHelpMenu.
+     *
+     * Example:
+     * \code
+     *
+     * void MyMainLevel::setupInterface()
+     * {
+     *   ..
+     *   menuBar()->addMenu( customHelpMenu() );
+     *   ..
+     * }
+     *
+     * void MyMainLevel::showAboutApplication()
+     * {
+     *   <activate your custom dialog>
+     * }
+     * \endcode
+     * @deprecated Since 5.0, use KHelpMenu
+     */
     KXMLGUI_DEPRECATED_VERSION(5, 0, "Use KHelpMenu")
-    virtual void showAboutApplication() {}
+    virtual void showAboutApplication()
+    {
+    }
 #endif
 
     /**
@@ -652,12 +656,16 @@ protected Q_SLOTS:
 protected:
     KMainWindow(KMainWindowPrivate &dd, QWidget *parent, Qt::WindowFlags f);
 
-    KMainWindowPrivate *const k_ptr;
-private:
-    Q_PRIVATE_SLOT(k_func(), void _k_slotSettingsChanged(int))
-    Q_PRIVATE_SLOT(k_func(), void _k_slotSaveAutoSaveSize())
-};
+    std::unique_ptr<KMainWindowPrivate> const k_ptr;
+    // KF6 TODO: change k_ptr to d_ptr, use normal Q_DECLARE_PRIVATE
 
+private:
+    Q_DECLARE_PRIVATE_D(k_ptr, KMainWindow)
+
+    Q_PRIVATE_SLOT(d_func(), void _k_slotSettingsChanged(int))
+    Q_PRIVATE_SLOT(d_func(), void _k_slotSaveAutoSaveSize())
+    Q_PRIVATE_SLOT(d_func(), void _k_slotSaveAutoSavePosition())
+};
 
 /**
  * @defgroup KXMLGUI_Session KXMLGUI Session Macros and Functions
@@ -672,10 +680,14 @@ private:
  *
  * @deprecated since 5.0, use kRestoreMainWindows() instead
  **/
-#define RESTORE(type) { int n = 1;\
-        while (KMainWindow::canBeRestored(n)){\
-            (new type)->restore(n);\
-            n++;}}
+#define RESTORE(type)                                                                                                                                          \
+    {                                                                                                                                                          \
+        int n = 1;                                                                                                                                             \
+        while (KMainWindow::canBeRestored(n)) {                                                                                                                \
+            (new type)->restore(n);                                                                                                                            \
+            n++;                                                                                                                                               \
+        }                                                                                                                                                      \
+    }
 #endif
 
 /**
@@ -722,7 +734,7 @@ private:
  * @see KMainWindow::restore()
  * @see KMainWindow::classNameOfToplevel()
  **/
-template <typename T>
+template<typename T>
 inline void kRestoreMainWindows()
 {
     for (int n = 1; KMainWindow::canBeRestored(n); ++n) {
@@ -731,7 +743,7 @@ inline void kRestoreMainWindows()
             (new T)->restore(n);
         }
     }
-} 
+}
 
 /**
  * Restores the last session.
@@ -741,7 +753,7 @@ inline void kRestoreMainWindows()
  * @tparam T1 explicit other toplevel widget class for disambiguation from base template
  * @tparam Tn Parameter pack to take 0..n further KMainWindows
  */
-template <typename T0, typename T1, typename ...Tn>
+template<typename T0, typename T1, typename... Tn>
 inline void kRestoreMainWindows()
 {
     kRestoreMainWindows<T0>();
